;****** VPI SDS Controller  ********

;  26-Sep-15		V 1.00         original entry
;  29-Sep-15		V 2.10		Add cal mode display, fac default and mem dump
;  19-Feb-17		V 2.20		Add hours reset mode
;
;  
;
;  
;                                
;   
;
;  
;
;   
;                                 
;                                 
;                                
;
;  
                   	include        "SDS.h"

;** set up vector table
                   	org            0                   ;reset
                   	sjmp           start

                   	org            3                   ;int0
                   	reti

                   	org            0bh                 ;timer 0
                   	ajmp           timer

                   	org            13h                 ;int 1
                   	reti                               ;not used

                   	org            1bh                 ;timer 1
                   	reti                               ;not used

                   	org            23h                 ;sio
                   	reti

;start of code segment

                   	org            30h
signon            	db             " SDS Controller "
                   	db             " v2.20 19-Feb-17"
start              	mov			p0,#0ffh			;init ports
				mov            p1,#80h
                   	mov            p3,#0ech

                   	mov            SP,#0fh			;set up stack

                   	mov            2ah,#0              ;init flags	
                   	mov			2bh,#0
                   	mov			MILEAGE_TMRHI,#180	;6 minute timer
                   	mov			MILEAGE_TMRLO,#200	
                   	mov			TM_DELAY,#60		;typematic initial delay
                   	mov			TM_SPEED,#16		;typematic initial speed
                   
                   	mov            SCON,#40h           ;mode 1: 8 bit UART
                   	mov            TMOD,#21h           ;T0=mode 1 :: T1=mode 2
                   	mov            TL0,#TMR_LOW        ;set up timer 0
                   	mov            TH0,#TMR_HIGH
                   	mov            TCON,#10h           ;start timer0
              		mov            IE,#82h             ;enable interupts
                   	mov            a,#25               ;250 mS delay
                   	acall          delay
				
start1             	mov			r0,#CYCLES_HI		;read from EPROM
				mov			r1,#3
				mov			r5,#1
				acall		rdram
                   	
				clr			C				;update power up count
				mov			a,CYCLES_LO
				add			a,#1
				da			a
				mov			CYCLES_LO,a
				jnc			start2
				mov			a,CYCLES_HI
				add			a,#1
				da			a
				mov			CYCLES_HI,a
start2			mov			r0,#CYCLES_HI		;save to EPROM
				mov			r1,#3
				mov			r5,#1
				acall		wram	
				
				jb			SPEED,start22		;check if SPEED pressed
				jnb			UP,start21		;yes, jump to cal mode if UP also pressed
				jb			DOWN,start3		;check for factory default
				ljmp			factory			;DOWN + SPEED pressed, do factory default
start21			ajmp			mem_dump			;UP + SPEED pressed, do mem dump

start22			jb			DOWN,start3		;check for cal mode
				jb			UP,start3			;
				ljmp			cal_mode			;UP + DOWN pressed, do cal mode
				
start3			acall          led_init            ;wake up display
				mov			a,#100			
				acall		delay
				acall		blank_disp					
				
boot				mov			r0,#FLAGS			;read 18 bytes into ram
				mov			r1,#0
				mov			r5,#9
				acall		rdram
				mov			a,CAL_LO			;div by 11,250 (2bf2h)
				mov			b,CAL_HI
				acall		load_ctr0			;send to ctr0 in U5
				
				mov			a,#0
				mov			b,#60h			;start at 60 Hz (6000)
				acall		load_ctr1			;send to ctr1 in U5
				
				mov			a,#0				;div by 4096 (1000h)
				mov			b,#10h			
				acall		load_ctr2			;send to ctr2 in U5
				ajmp			main_stby
				
boot1			mov			a,FLAGS			;check current speed
				jz			boot45
				mov			a,LOW_33			;33 RPM
				mov			b,HIGH_33
				mov			r0,#HIGH_33
				mov			r1,#ATTN_33
				mov			DISP8,#0fbh
				sjmp			boot2
boot45			mov			a,LOW_45			;45 RPM
				mov			b,HIGH_45
				mov			r0,#HIGH_45
				mov			r1,#ATTN_45
				mov			DISP8,#4
boot2			acall		load_ctr1			;send freq to ctr 1
				clr			SINE_DIS			;turn on output

				acall		disp_freq			;display current freq	
				mov			DISP7,#0b0h		;display 115VAC
				mov			DISP6,#0b0h
				mov			DISP5,#0d7h										
				acall		write_disp		;display freq data
				mov			p1,#0			;start at full power
				
				jnb			SPEED,$			;wait until unkeyed
				
				clr			VOLT_TOT
				mov			a,@r1			;check attenuation
				jz			main				;skip if no attn
				mov			VOLT_TIMER,#250	;set timer
				clr			KEY_SHORT
				clr			KEY_LONG
				
main				jnb			KEY_SHORT,main1	;check for speed change
				clr			KEY_SHORT
				mov			a,FLAGS
				jz			main01			;jump if 45 RPM
				acall		chg33_45			;ramp freq up
				mov			FLAGS,#0			;33 so change to 45
				sjmp			main02
main01			acall		chg45_33			;ramp freq down
				mov			FLAGS,#2			;45 so change to 33
main02			mov			r0,#FLAGS
				mov			r1,#0
				mov			r5,#1
				acall		wram				
				ajmp			boot1
				
main1			jnb			KEY_LONG,main2		;check for standby mode
main_stby			clr			KEY_LONG
				acall		blank_disp		;yes, go to standby
				mov			DISP4,#0d7h		
				mov			DISP3,#8fh
				mov			DISP2,#9fh
				mov			DISP1,#0b7h
				acall		write_disp
				setb			SINE_DIS			;turn off output
				
				jnb			SPEED,$			;wait until unkeyed
				mov			r7,#50
				acall		soft_delay
				
				jnb			KEY_SHORT,$		;sleep until keyed
				clr			KEY_SHORT
				mov			a,FLAGS			;check for start in 45 RPM
				jnz			main11			;33, so just go
				clr			a
				acall		disp_hex1
				acall		chg33_45			;45, start at 33 and ramp up			
main11			ajmp			boot1
				
				
main2			jnb			VOLT_TOT,main3		;check for reduce voltage												
				mov			r1,#ATTN_33
				mov			a,FLAGS
				jnz			main21			;jump if 33 RPM
				inc			r1
main21			mov			a,@r1			;get attn				
				mov			r4,a				;save count
				mov			r5,#1
main22			mov			a,r5				
				acall		write_attn
				acall		write_disp
				mov			a,r5
				rl			a
				rl			a
				rl			a
				mov			p1,a
				mov			a,#5
				acall		delay
				inc			r5
				djnz			r4,main22
				clr			VOLT_TOT
								
main3			jb			UP,main4			;check if up arrow pressed
main30			mov			r0,#LOW_33		;yes get pointer to data
				mov			LIMIT,#65h		;set max for 33
				mov			a,FLAGS
				jnz			main31
				mov			r0,#LOW_45
				mov			LIMIT,#86h		;set max for 45
main31			mov			a,@r0
				jnz			main315			;check for max
				dec			r0
				mov			a,@r0
				inc			r0
				cjne			a,LIMIT,main315		
				sjmp			main325
main315			mov			a,@r0
				add			a,#1				;increase by .01 Hz
				da			a				
				mov			@r0,a
				jnc			main32			;check for over flow
				dec			r0		
				mov			a,@r0
				add			a,#1				;yes, add 1 to Hz	
				da			a
				mov			@r0,a
main32			acall		disp_freq			;send to display
				acall		send_freq			;send to U5				
main325			setb			TYPEMATIC
				clr			TM_TOT
main33			jb			TM_TOT,main34		;wait for timeout
				jnb			UP,main33			;or release
				clr			TYPEMATIC			;released, reset typematic effect	
				mov			TM_DELAY,#60		;reset typematic delay	
				mov			TM_SPEED,#16		;reset repeat speed
				sjmp			main36
main34			mov			a,TM_SPEED		;load current repeat rate
				xrl			a,#5				;check if min
				jz			main35
				dec			TM_SPEED 			;speed it up
main35			mov			TM_DELAY,TM_SPEED	;load delay	
				sjmp			main30	

main36			mov			r7,#50
				acall		soft_delay
				
				mov			r0,#HIGH_33		;save in EPROM
				mov			r1,#1
				mov			a,FLAGS
				jnz			main37
				mov			r0,#HIGH_45
				mov			r1,#2
main37			mov			r5,#1
				acall		wram
				
				
main4			jb			DOWN,main5
main40			mov			r0,#LOW_33		;yes get pointer to data
				mov			LIMIT,#55h
				mov			a,FLAGS
				jnz			main41
				mov			r0,#LOW_45
				mov			LIMIT,#76h
main41			mov			a,@r0
				jnz			main415			;check for minimum value
				dec			r0
				mov			a,@r0
				inc			r0
				cjne			a,LIMIT,main415		
				sjmp			main425
main415			mov			a,@r0
				add			a,#99h			;decrease by .01 Hz
				da			a				
				mov			@r0,a
				jc			main42			;check for over flow
				dec			r0		
				mov			a,@r0
				add			a,#99h			;yes, subtract 1 from Hz	
				da			a
				mov			@r0,a
main42			acall		disp_freq			;send to display
				acall		send_freq			;send to U5
main425			setb			TYPEMATIC
				clr			TM_TOT
main43			jb			TM_TOT,main44		;wait for timeout
				jnb			DOWN,main43		;or release
				clr			TYPEMATIC			;released, reset typematic effect	
				mov			TM_DELAY,#60		;reset typematic delay	
				mov			TM_SPEED,#16		;reset repeat speed
				sjmp			main46
main44			mov			a,TM_SPEED		;load current repeat rate
				xrl			a,#5				;check if min
				jz			main45
				dec			TM_SPEED 			;speed it up
main45			mov			TM_DELAY,TM_SPEED	;load delay	
				sjmp			main40				
				
main46			mov			r7,#50
				acall		soft_delay
				
				mov			r0,#HIGH_33		;save in EPROM
				mov			r1,#1
				mov			a,FLAGS
				jnz			main47
				mov			r0,#HIGH_45
				mov			r1,#2
main47			mov			r5,#1
				acall		wram
				
				
main5			jnb			MILEAGE_TOT,main6	;check for 6 min time out
				clr			MILEAGE_TOT
				mov			r0,#HOURS_LSB		;update mileage	
				mov			r2,#4			;4 locations max			
main51			clr			C
				mov			a,@r0
				add			a,#1
				da			a
				mov			@r0,a
				jnc			main52
				dec			r0
				djnz			r2,main51
main52			mov			r0,#HOURS_MSB		;save in EPROM
				mov			r1,#4
				mov			r5,#2				
				acall		wram
				
main6			ajmp			main												
				
				
chg33_45			mov			p1,#0
				mov			DISP8,#4
				mov			DISP7,#0b0h		;display 115VAC
				mov			DISP6,#0b0h
				mov			DISP5,#0d7h
				mov			a,HIGH_45			;get difference in Hz
				anl			a,#0f0h			;change to binary
				swap			a
				mov			b,#10
				mul			ab
				mov			HEX2,a
				mov			a,HIGH_45
				anl			a,#0fh
				add			a,HEX2
				mov			HEX2,a			;HEX2=45 freq (Hz) in binary
								
				mov			a,HIGH_33			;get difference in Hz
				anl			a,#0f0h			;change to binary
				swap			a
				mov			b,#10
				mul			ab
				mov			HEX1,a
				mov			a,HIGH_33
				anl			a,#0fh
				add			a,HEX1
				mov			HEX1,a			;HEX1=33 freq (Hz) in binary
				clr			C
				mov			a,HEX2
				subb			a,HEX1
				mov			r5,a				;R5= freq45-freq33 in Hz
				
				mov			HEX2,HIGH_33		;start at 33
				mov			HEX1,LOW_33
chg33_45_loop		mov			a,HEX2
				add			a,#1
				da			a
				mov			HEX2,a
				mov			b,a
				mov			a,HEX1
				acall		load_ctr1			;send freq to U5
				mov			a,HEX2
				acall		disp_hex2
				acall		write_disp		;send interim freq to display
				mov			r7,#100
				acall		soft_delay
				djnz			r5,chg33_45_loop	
				ret
				
	
chg45_33			mov			p1,#0
				mov			DISP8,#0fbh
				mov			DISP7,#0b0h		;display 115VAC
				mov			DISP6,#0b0h
				mov			DISP5,#0d7h
				mov			a,HIGH_45			;get difference in Hz
				anl			a,#0f0h			;change to binary
				swap			a
				mov			b,#10
				mul			ab
				mov			HEX2,a
				mov			a,HIGH_45
				anl			a,#0fh
				add			a,HEX2
				mov			HEX2,a			;HEX2=45 freq (Hz) in binary
								
				mov			a,HIGH_33			;get difference in Hz
				anl			a,#0f0h			;change to binary
				swap			a
				mov			b,#10
				mul			ab
				mov			HEX1,a
				mov			a,HIGH_33
				anl			a,#0fh
				add			a,HEX1
				mov			HEX1,a			;HEX1=33 freq (Hz) in binary
				clr			C
				mov			a,HEX2
				subb			a,HEX1
				mov			r5,a				;R5= freq45-freq33 in Hz
				
				mov			HEX2,HIGH_45		;start at 45
				mov			HEX1,LOW_45
chg45_33_loop		mov			a,HEX2
				add			a,#99h			;subtract 1 BCD
				da			a
				mov			HEX2,a
				mov			b,a
				mov			a,HEX1
				acall		load_ctr1			;send freq to U5
				mov			a,HEX2
				acall		disp_hex2
				acall		write_disp		;send interim freq to display
				mov			r7,#100
				acall		soft_delay
				djnz			r5,chg45_33_loop
				ret				

send_freq			mov			a,FLAGS			;send 16 bit freq to U5 ctr1
				jz			send_freq1		;jump if 45			
				mov			b,HIGH_33
				mov			a,LOW_33
				sjmp			send_freq2
send_freq1		mov			b,HIGH_45
				mov			a,LOW_45
send_freq2		acall		load_ctr1			;send to U5
				ret
				
				
											
								
mem_dump			acall		blank_disp
				mov			r0,#FLAGS			;read 24 bytes into ram
				mov			r1,#0
				mov			r5,#12
				acall		rdram
				mov			DISP6,#0cfh
				mov			DISP5,#0edh		;display E2
				acall		write_disp
				jnb			SPEED,$			;wait for unkey
				
				mov			r1,#FLAGS			;display mem location 1 byte at a time
				mov			r5,#24
mem_dump1			jnb			SPEED,$
				mov			r7,#100
				acall		soft_delay				
				mov			a,#FLAGS			;get address
				cpl			a
				inc			a
				add			a,r1
				mov			r7,a
				anl			a,#0f0h
				swap			a
				acall		dispconv
				mov			DISP6,a			;DISP6/5 shows address 00-0Ch
				mov			a,r7
				anl			a,#0fh
				acall		dispconv
				mov			DISP5,a
				mov			a,@r1
				anl			a,#0f0h
				swap			a
				acall		dispconv
				mov			DISP2,a			;DISP2/1 shows contents in hex
				mov			a,@r1
				anl			a,#0fh
				acall		dispconv
				mov			DISP1,a
				jb			SPEED,$
				acall		write_disp
				inc			r1
				mov			r7,#100
				acall		soft_delay
				djnz			r5,mem_dump1
				
				

				sjmp			$                  	
                   
											;delay 10mSec x ACC
delay              	setb           PSW.3               ;select RB1
                   	mov            TL0,#TMR_LOW        ;synch timer
                   	mov            TH0,#TMR_HIGH
                   	jnz            delay1              ;trap a=0
                   	inc            a	               ;a=1
delay1             	mov            r0,a
                   	clr            PSW.3               ;select RB0
                   	clr            SHORT               ;reset short flag
                   	jnb            SHORT,$             ;wait here until timeout
                   	ret

soft_delay         	mov            r2,#122             ;1mSec X R7 @ 7.3728 MHz
sd1        	 	nop
                   	nop
                   	nop
                   	djnz           r2,sd1		     ;5 cyc X 122
                   	djnz           r7,soft_delay       ;+4=614 x 1.6276 Sec/cyc
                   	ret

;*************************
; timer int every 10 mS
;*************************

timer              	push           PSW                 ;save status word
				push			ACC
                   	setb			PSW.3             	;select RB1
                   	mov            TL0,#TMR_LOW        ;reload timer
                   	mov            TH0,#TMR_HIGH

timer1             	mov            a,r0                ;short timer active?
                   	jz             timer2              ;no--continue
                   	djnz           r0,timer2           ;yes--update
                   	setb           SHORT               ;timed out so flag it
timer2             	clr            a                   ;long delay active?
                   	orl            a,r2
                   	orl            a,r3
                   	jz             timer3              ;no--continue
                   	dec            r3                  ;yes--update
                   	cjne           r3,#0ffh,timer21
                   	dec            r2                  ;update if underflow
timer21            	clr            a                   ;end of long delay?
                   	orl            a,r2
                   	orl            a,r3
                   	jnz            timer3              ;no-continue
                   	setb           LONG                ;yes flag it

timer3			jnb			TYPEMATIC,timer4	;check if active
				djnz			TM_DELAY,timer4
				setb			TM_TOT		
				
timer4			jnb			SPEED,timer43		;jump if pressed
				jnb			KEYED,timer5		;exit if not prev keyed
				clr			KEYED
				jnb			KEY_LONG,timer41
				clr			KEY_LONG
				sjmp			timer5
timer41			mov			a,KEY_DELAY		;check time key was pressed
				cjne			a,#70,timer42
timer42			jnc			timer5			;=<5 counts so skip
				setb			KEY_SHORT			;> 5 counts, flag it
				sjmp			timer5
timer43			jb			KEYED,timer44		;jump if already pressed
				setb			KEYED			;no, flag for next time
				mov			KEY_DELAY,#75		;load with 750mS
				sjmp			timer5
timer44			jb			KEY_LONG,timer5	;skip if already flagged
				djnz			KEY_DELAY,timer5	;update counter
				setb			KEY_LONG
				
timer5			mov			a,VOLT_TIMER		;check if active
				jz			timer6
				djnz			VOLT_TIMER,timer6	;yes, update
				setb			VOLT_TOT			;flag if expired
				
timer6			djnz			MILEAGE_TMRLO,timer7	
				mov			MILEAGE_TMRLO,#200
				djnz			MILEAGE_TMRHI,timer7
				mov			MILEAGE_TMRHI,#180										
				setb			MILEAGE_TOT		;36,000 x 10mS=6minute=0.1 Hour
				
timer7			nop	
				
done               	pop            ACC
                   	pop            PSW                 ;restore psw
                   	reti                               ;done!


                   	
                   	
                   								;send 16bits B:A to ctr0 in U5
load_ctr0			setb			A1				;addrss ctrl reg
				setb			A0
				clr			TIMER_CS
				mov			p2,#36h			;ctr0, 2 bytes, mode 3, binary
				clr			TIMER_WR			;latch it
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				
				clr			A1
				clr			A0				;address ctr 0
				clr			TIMER_CS
				mov			P2,a				;LSB first
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				nop
				nop
				clr			TIMER_CS
				mov			P2,b				;MSB next
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS                   	
                   	ret
                   	
                   								;send 16bits B:A to ctr1 in U5
load_ctr1			setb			A1				;addrss ctrl reg
				setb			A0
				clr			TIMER_CS
				mov			p2,#77h			;ctr1, 2 bytes, mode 3, bcd mode
				clr			TIMER_WR			;latch it
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				
				clr			A1				;address ctr 1
				clr			TIMER_CS
				mov			P2,a				;LSB first
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				nop
				nop
				clr			TIMER_CS
				mov			P2,b				;MSB next
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS                   	
                   	ret
                   	
                   	
                   								;send 16bits B:A to ctr2 in U5
load_ctr2			setb			A1				;addrss ctrl reg
				setb			A0
				clr			TIMER_CS
				mov			p2,#0b6h			;ctr2, 2 bytes, mode 3, binary
				clr			TIMER_WR			;latch it
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				
				clr			A0				;address ctr 2
				clr			TIMER_CS
				mov			P2,a				;LSB first
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS
				nop
				nop
				clr			TIMER_CS
				mov			P2,b				;MSB next
				clr			TIMER_WR
				nop
				setb			TIMER_WR
				setb			TIMER_CS                   	
                   	ret                   	                   	
                   	
disp_hex2			push			ACC				;display acc as 2 hex digits
				anl			a,#0f0h			;at location DISP4/3
				swap			a
				acall		dispconv
				mov			DISP4,a
				pop			ACC
				anl			a,#0fh
				acall		dispconv
				clr			ACC.7
				mov			DISP3,a
				ret
				
disp_hex1			push			ACC				;display acc as 2 hex digits
				anl			a,#0f0h			;at location DISP2/1
				swap			a
				acall		dispconv
				mov			DISP2,a
				pop			ACC
				anl			a,#0fh
				acall		dispconv
				mov			DISP1,a
				ret				
				                   	
write_attn		mov			b,#3				;display 3 digit voltage reading
				mul			ab				;at location DISP7/6/5
				mov			r7,a				;save pointer
				mov			dptr,#attn_table
				movc			a,@a+dptr
				mov			DISP7,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP6,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP5,a
				ret
attn_table		db			0b0h,0b0h,0d7h,0b0h,0fbh,0ffh,0b0h,0fbh,0edh, 80h,0f7h,0dfh
				db			 80h,0f7h,0fbh, 80h,0ffh,0b6h, 80h,0f0h,0ffh, 80h,0f0h,0edh				                   	
                   	


;*******************************
;* LEDINIT INITILIZE THE LED
;*******************************

led_init           	mov			r3,#8			;write 7 digits
				mov			r0,#DISP8			
				mov			dptr,#mess1
led_init1			clr			a
				movc			a,@a+dptr
				mov			@r0,a
				inc			dptr
				dec			r0
				djnz			r3,led_init1
				acall		write_disp				
				ret


mess1			db			80h,0eeh,9eh,9dh,0cfh,9ch,90h,0beh		



											;Display driver mulitplexes 8 LED displays
											;DISP7-DISP1 7 seg digits (DISP1 right most)
											;DISP8 controls 33/45 RPM icons
write_disp		push			REG0				;write 8 digits of LED display
				push			REG3				;from mem locations DISP8-DISP1
				setb			MODE				;write control reg
				mov			p2,#0b0h			;no decode, seq wr, normal op
				setb			LED_WR
				nop
				clr			LED_WR
				
				clr			MODE
				mov			r3,#8
				mov			r0,#DISP1			;point to beginning
write_disp1		mov			a,@r0
				mov			p2,a
				setb			LED_WR
				nop
				clr			LED_WR
				inc			r0
				djnz			r3,write_disp1
				pop			REG3
				pop			REG0
				ret
				
				
blank_disp		mov			r0,#DISP1			;blank display
				mov			a,#80h			
				mov			r3,#8
blank1			mov			@r0,a
				inc			r0
				djnz			r3,blank1
				acall		write_disp	
				ret		
				
disp_freq			mov			r0,#HIGH_33		;display current freq at DISP4-DISP1
				mov			a,FLAGS
				jnz			disp_freq1
				mov			r0,#HIGH_45
disp_freq1		mov			a,@r0				
				acall		disp_hex2			;send to LED display
				inc			r0
				mov			a,@r0
				acall		disp_hex1
				acall		write_disp
				ret					


;**********************************
;* display converter for LED module
;**********************************

dispconv           	mov            dptr,#disptable     ;get offset
                   	movc           a,@a+dptr           ;read from table
                   	ret
disptable          	db             0fbh,0b0h,0edh,0f5h,0b6h,0d7h,0dfh,0f0h,0ffh,0f7h,0feh,9fh,0cbh,0bdh,0cfh,0ceh 


;***************************************
; rdram reads EPROM into ram and flags
; R0=RAM pointer
; R1=EEPROM address
; R5=16 bit words to read
;***************************************

rdram              setb           	EE_CE
                   mov            	a,r1                ;get address
                   orl            	a,#80h              ;set bit 7 for instr
                   acall          	instr
rdram1             acall          	read                ;read 1st word
                   mov            	@r0,a               ;save it
                   inc            	r0                  ;point to next
                   acall          	read                ;read 2nd word
                   mov            	@r0,a               ;save it
                   inc            	r0                  ;move ram pointer
                   inc            	r1                  ;increment eprom address
                   clr            	EE_CE
                   djnz           	r5,rdram            ;do again
                   ret

;***************************************
; wram writes ram and flags into EPROM
; R0=RAM pointer
; R1=EEPROM address
; R5=16 bit words to write
;***************************************

wram               setb           	EE_CE
                   mov            	a,#30h              ;wren instr
                   acall          	instr
                   clr            	EE_CE
wram1              setb           	EE_CE
                   mov            	a,r1                ;get eprom address
                   orl            	a,#40h              ;set op code
                   acall          	instr
                   mov            	a,@r0               ;write 1st word
                   acall          	write
                   inc            	r0
                   mov            	a,@r0               ;write 2nd word
                   acall          	write
                   inc            	r0                  ;point to next ram address
                   inc            	r1                  ;point to next eprom addr
                   clr            	EE_CE               ;clock ce
                   clr            	SDATA               ;bring DI low
                   setb           	EE_CE

wram2              jnb            	DATA_IN,wram2       ;wait here until program
                   clr            	EE_CE               ;takes
                   djnz           	r5,wram1            ;do twice

                   setb           	EE_CE
                   clr            	a                   ;write disable instruction
                   acall          	instr
                   clr            	EE_CE
                   ret


;****************************************
; read returns 8 bits from EPROM in acc
;****************************************

read               mov            	r4,#8
read1              setb           	SCK
                   clr            	SCK
                   mov            	C,DATA_IN
	              rlc            	a
                   djnz           	r4,read1
                   ret

;***************************
; write sends acc to EPROM
;***************************

write              mov            	r4,#8
write1             rlc            	a
                   mov			SDATA,C
	              setb           	SCK                 ;clock it
                   clr            	SCK
                   djnz           	r4,write1           ;do 8 times
                   ret

;***************************************************************
; instr sends dummy zero + opcode  from accumulator to EPROM
;***************************************************************

instr              mov            	r4,#8
                   clr            	SDATA               ;write dummy 0
                   setb           	SCK
                   clr            	SCK
                   setb           	SDATA               ;write start bit
                   setb           	SCK
                   clr            	SCK

instr1             rlc            	a
                   mov			SDATA,C
                   setb           	SCK                 ;clock it
                   clr            	SCK
                   djnz           	r4,instr1           ;do 4 times
                   ret



                   	
                   	
hex_to_bcd         	mov            BCD1,#0             ;convert 16bit hex2:hex1
                   	mov            BCD2,#0             ;into BCD3:BCD2:BCD1
                   	mov            BCD3,#0             ;clear values
                   	mov            r3,#16              ;16 bits to shift

hex_to_bcd_loop    	mov            r4,#3               ;check 3 bytes
                   	mov            r0,#BCD1            ;point to first
check_lo_nibble    	mov            a,@r0               ;check LSB
                   	anl            a,#0fh              ;mask it
                   	add            a,#0fbh             ;>5?
                   	jnc            check_hi_nibble
                   	mov            a,@r0               ;yes, adjust
                   	add            a,#3
                   	mov            @r0,a

check_hi_nibble    	mov            a,@r0
                   	add            a,#0b0h
                   	jnc            check_next
                   	mov            a,@r0
                   	add            a,#30h
                   	mov            @r0,a
check_next         	dec            r0
                   	djnz           r4,check_lo_nibble

                   	mov            r0,#HEX1            ;point to array
                   	mov            r4,#5               ;shift 5 bytes
shift_bcd          	mov            a,@r0               ;get data
                   	rlc            a                   ;shift it
                   	mov            @r0,a               ;store it
                   	dec            r0
                   	djnz           r4,shift_bcd

                   	djnz           r3,hex_to_bcd_loop
                   	ret
                   	
                   							    ;convert BCD3:BCD2:BCD1
bcd_to_hex         mov            HEX2,#0             ;into hex2:hex1
                   mov            HEX1,#0             ;clear values
                   mov            r3,#16              ;16 bits to shift

bcd_to_hex_loop    mov            r4,#5               ;shift array
                   mov            r0,#BCD3            ;point to 1st
                   clr            C
shift_hex          mov            a,@r0               ;get byte
                   rrc            a                   ;shift it
                   mov            @r0,a               ;save it
                   inc            r0                  ;point to next
                   djnz           r4,shift_hex

                   mov            r4,#3               ;check 3 bytes
                   mov            r0,#BCD3            ;point to first
chk_high_nibble    mov            a,@r0
                   cjne           a,#60h,chn1         ;>5xh?
chn1               jc             chk_low_nibble      ;no, check low nibble
                   clr            C                   ;yes, adjust
                   subb           a,#30h              ;subtract 3 from hi nibble
                   mov            @r0,a               ;save it

chk_low_nibble     anl            a,#0fh              ;mask low nibble
                   cjne           a,#6,cln1           ;>5?
cln1               jc             chk_next            ;no, go to next byte
                   mov            a,@r0               ;yes, get byte
                   clr            C
                   subb           a,#3                ;and subtract 3
                   mov            @r0,a               ;save it
chk_next           inc            r0                  ;point to next
                   djnz           r4,chk_high_nibble

                   djnz           r3,bcd_to_hex_loop
                   ret    
                   
                   org			800h
                   
cal_mode			mov			r0,#FLAGS
				mov			r1,#0
				mov			r5,#9
				lcall		rdram			;get current state from EPROM
				mov			a,CAL_LO			;div by 11,250 (2bf2h)
				mov			b,CAL_HI
				lcall		load_ctr0			;send to U5 ctr0
				
				mov			a,#0
				mov			b,#90h			;start at 90 Hz (9000)
				lcall		load_ctr1			;send to U5 ctr 1
				
				mov			a,#0				;div by 4096 (1000h)
				mov			b,#10h			
				lcall		load_ctr2			;send to U5 ctr2
				
				lcall		blank_disp
				mov			DISP7,#0cbh		;display "CAL"
				mov			DISP6,#0feh
				mov			DISP5,#8bh
				lcall		write_disp
				
				jnb			KEY_SHORT,$		;wait for short press of Freq
				clr			KEY_SHORT
				mov			DISP5,#80h
				
				mov			a,#90h
				lcall		disp_hex2			;display 90.00 Hz
				clr			a       
				lcall		disp_hex1
				mov			DISP7,#0cbh		;display C1
				mov			DISP6,#0b0h
				lcall		write_disp
				clr			SINE_DIS			;turn on output
				
cal_mode1			jb			KEY_SHORT,cal_mode2	;first SPD press, skip adjust mode
				jnb			UP,cal_mode11		;check for up/down press
				jb			DOWN,cal_mode1		;loop here until key is pressed
				
cal_mode11		jnb			UP,cal_mode12		;jump if UP pressed
				clr			C				;down pressed	
				mov			a,CAL_LO			;increment 16 bit cal value
				add			a,#1	
				mov			CAL_LO,a
				jnc			cal_mode13
				inc			CAL_HI
				sjmp			cal_mode13
				
cal_mode12		clr			C
				mov			a,CAL_LO			;decrement 16 bit cal value
				subb			a,#1	
				mov			CAL_LO,a
				jnc			cal_mode13
				dec			CAL_HI
				
cal_mode13		mov			HEX2,CAL_HI		;display current ref counter value
				mov			HEX1,CAL_LO
				lcall		hex_to_bcd
				mov			a,BCD3
				anl			a,#0fh
				lcall		dispconv
				mov			DISP5,a
				mov			a,BCD2
				lcall		disp_hex2
				mov			a,BCD1
				lcall		disp_hex1
				mov			a,DISP3
				setb			ACC.7
				mov			DISP3,a
				lcall		write_disp

				mov			a,CAL_LO			;send new value to U5
				mov			b,CAL_HI
				lcall		load_ctr0
				jnb			UP,$
				jnb			DOWN,$			;wait for release
				mov			r7,#50
				lcall		soft_delay

cal_mode14		jb			KEY_SHORT,cal_mode15  ;save on SPD press
				jnb			UP,cal_mode11
				jnb			DOWN,cal_mode11
				sjmp			cal_mode14
				
cal_mode15		mov			r0,#CAL_HI
				mov			r1,#7
				mov			r5,#1
				lcall		wram				
				
cal_mode2			clr			KEY_SHORT
				setb			SINE_DIS
				mov			DISP6,#0edh		;display C2
				mov			DISP5,#80h
				mov			a,CYCLES_HI
				lcall		disp_hex2
				mov			a,DISP3
				setb			ACC.7
				mov			DISP3,a			;turn off decimal point
				mov			a,CYCLES_LO
				lcall		disp_hex1
				lcall		write_disp
				
				jnb			KEY_SHORT,$
				
cal_mode3			clr			KEY_SHORT
				mov			DISP6,#0f5h		;display C3A
				mov			DISP5,#0feh
				mov			r0,#HOURS_MSB
				mov			a,@r0
				lcall		disp_hex2
				mov			a,DISP3
				setb			ACC.7
				mov			DISP3,a
				inc			r0
				mov			a,@r0
				lcall		disp_hex1
				lcall		write_disp
				
				jnb			KEY_SHORT,$
				
				clr			KEY_SHORT		
				mov			DISP5,#9Fh		;display C3b
				inc			r0
				mov			a,@r0
				lcall		disp_hex2
				mov			a,DISP3
				setb			ACC.7
				mov			DISP3,a
				inc			r0
				mov			a,@r0
				lcall		disp_hex1
				mov			a,DISP2			;turn on decimal point
				clr			ACC.7
				mov			DISP2,a
				lcall		write_disp
				
cal_mode31		jb			KEY_SHORT,cal_mode4	;wait for speed pressed
				jnb			KEY_LONG,cal_mode31	;or long press
				lcall		blank_disp
				clr			STATE
				mov			DISP7,#0cbh		;display CLr
				mov			DISP6,#8bh
				mov			DISP5,#8ch
cal_mode32		jb			STATE,cal_mode33
				mov			DISP4,#80h		;display _no
				mov			DISP3,#9ch
				mov			DISP2,#9dh
				sjmp			cal_mode34
cal_mode33		mov			DISP4,#0b7h		;display YES
				mov			DISP3,#0cfh				
				mov			DISP2,#0d7h
cal_mode34		lcall		write_disp
				jnb			SPEED,$			;wait for unkey
				jnb			UP,$
				jnb			DOWN,$
				clr			KEY_SHORT						
				clr			KEY_LONG
				mov			r7,#50
				lcall		soft_delay

cal_mode35		jnb			UP,cal_mode36
				jnb			DOWN,cal_mode36
				jnb			KEY_LONG,cal_mode35
				jnb			STATE,cal_mode4	;skip zero mode if state=no				
				mov			38h,#0			;clear hours counter
				mov			39h,#0
				mov			3ah,#0
				mov			3bh,#0
				mov			3ch,#0
				mov			3dh,#0
				mov			r0,#HOURS_MSB
				mov			r1,#4
				mov			r5,#3
				lcall		wram				;save it to EERPOM
				sjmp			cal_mode4
				
cal_mode36		cpl			STATE
				sjmp			cal_mode32				
				
cal_mode4			clr			KEY_LONG
				clr			KEY_SHORT			;short press, skip clear mode
				mov			DISP6,#0b6h
				mov			DISP5,#80h		;display C4
				mov			DISP4,#80h
cal41			mov			a,ATTN_33
				mov			b,#3
				mul			ab
				mov			r7,a				;save pointer
				mov			dptr,#attn_table
				movc			a,@a+dptr
				mov			DISP3,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP2,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP1,a
				lcall		write_disp
				jnb			UP,$
				jnb			DOWN,$
				mov			r7,#100
				lcall		soft_delay				
				
cal42			jb			KEY_SHORT,cal5		;wait for keypress
				jnb			DOWN,cal43
				jb			UP,cal42
				
				mov			a,ATTN_33			;down pressed increase attn
				dec			a
				anl			a,#7
				mov			ATTN_33,a
				sjmp			cal41
				
cal43			mov			a,ATTN_33				
				inc			a
				anl			a,#7
				mov			ATTN_33,a
				sjmp			cal41
				
cal5				mov			r0,#ATTN_33
				mov			r1,#8
				mov			r5,#1
				lcall		wram

				clr			KEY_SHORT				
				mov			DISP6,#0d7h		;display C5
cal51			mov			a,ATTN_45
				mov			b,#3
				mul			ab
				mov			r7,a				;save pointer
				mov			dptr,#attn_table
				movc			a,@a+dptr
				mov			DISP3,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP2,a
				mov			a,r7
				inc			dptr
				movc			a,@a+dptr
				mov			DISP1,a
				lcall		write_disp
				
				jnb			UP,$
				jnb			DOWN,$
				mov			r7,#100
				lcall		soft_delay				
				
cal52			jb			KEY_SHORT,cal6		;wait for keypress
				jnb			DOWN,cal53
				jb			UP,cal52
				
				mov			a,ATTN_45			;down pressed increase attn
				dec			a
				anl			a,#7
				mov			ATTN_45,a
				sjmp			cal51
				
cal53			mov			a,ATTN_45				
				inc			a
				anl			a,#7
				mov			ATTN_45,a
				sjmp			cal51
				
cal6				mov			r0,#ATTN_33
				mov			r1,#8
				mov			r5,#1
				lcall		wram
				
				
				clr			KEY_SHORT
				mov			DISP6,#0dfh		;display C6
				mov			dptr,#42h
				clr			a
				movc			a,@a+dptr
				add			a,#0d0h			;convert from ASCII to binary
				lcall		disp_hex2
				mov			DISP4,#99h		;display 'v'
				mov			dptr,#44h			;get minor rev
				clr			a
				movc			a,@a+dptr
				add			a,#0d0h
				mov			b,#10
				mul			ab
				mov			b,a
				inc			dptr
				clr			a
				movc			a,@a+dptr
				add			a,#0d0h
				add			a,b
				lcall		disp_hex1
				lcall		write_disp
				
				jnb			KEY_SHORT,$
				mov			a,#50
				lcall		delay
				clr			KEY_SHORT
				ljmp			start3
				
factory			lcall		blank_disp
				mov			DISP7,#0ceh		;disp FAC
				mov			DISP6,#0feh
				mov			DISP5,#0cbh
				lcall		write_disp
				mov			r0,#80h
				mov			r1,#0
				mov			r5,#9			;get 18 bytes current data
				lcall		rdram
				
				jnb			DOWN,$
				jnb			SPEED,$			;wait for release
				
				mov			r0,#92h			;blank high addresses
				mov			a,#0ffh
				mov			r5,#110
factory1			mov			@r0,a			
				inc			r0
				djnz			r5,factory1
				
				mov			r0,#80h			;load 18 bytes default data
				mov			dptr,#fac_table
				mov			r5,#18
factory2			cjne			r0,#86h,factory21	;don't overwrite power ups
				sjmp			factory3
factory21			cjne			r0,#87h,factory22
				sjmp			factory3
factory22			cjne			r0,#88h,factory23	;or hours
				sjmp			factory3
factory23			cjne			r0,#89h,factory24
				sjmp			factory3
factory24			cjne			r0,#8ah,factory25
				sjmp			factory3
factory25			cjne			r0,#8bh,factory26
				sjmp			factory3
factory26			clr			a
				movc			a,@a+dptr
				mov			@r0,a
factory3			inc			r0
				inc			dptr
				djnz			r5,factory2
				
				mov			r0,#80h			;write the array to EPROM
				mov			r1,#0
				mov			r5,#64				
				lcall		wram
				
				mov			a,#50
				lcall		delay
				
				lcall		blank_disp
				mov			DISP4,#0bdh		;display "donE"
				mov			DISP3,#9dh
				mov			DISP2,#9ch
				mov			DISP1,#0cfh
				lcall		write_disp
				
				clr			KEY_SHORT
				jnb			KEY_SHORT,$
				mov			a,#50
				lcall		delay
				clr			KEY_SHORT
				ljmp			start

						    ;33RPM   60.00Hz 81.00Hz                                   11,250   78V 102V
fac_table			db			02h,00h,60h,00h,81h,00h,00h,00h,00h,00h,00h,00h,0a2h,0e9h,2bh,0f2h,06h,02h
				
				
				
				
				
				
				
														               	


